@@ -20,7 +20,7 @@ extern "C" {
 
 
 #ifndef M_PI
-#define	M_PI	3.1415926535897932384626433832795
+#define M_PI    3.1415926535897932384626433832795
 #endif
 
 
@@ -29,18 +29,18 @@ extern "C" {
 /*
  * Image data organization:
  *
- * mode	    bytes	byte order
+ * mode     bytes       byte order
  * -------------------------------
- * 1	    1		1
- * L	    1		L
- * P	    1		P
+ * 1        1           1
+ * L        1           L
+ * P        1           P
  * I        4           I (32-bit integer, native byte order)
  * F        4           F (32-bit IEEE float, native byte order)
- * RGB	    4		R, G, B, -
- * RGBA	    4		R, G, B, A
- * CMYK	    4		C, M, Y, K
- * YCbCr    4		Y, Cb, Cr, -
- * Lab      4       L, a, b, -
+ * RGB      4           R, G, B, -
+ * RGBA     4           R, G, B, A
+ * CMYK     4           C, M, Y, K
+ * YCbCr    4           Y, Cb, Cr, -
+ * Lab      4           L, a, b, -
  *
  * experimental modes (incomplete):
  * LA       4           L, -, -, A
@@ -75,29 +75,35 @@ typedef struct ImagingPaletteInstance* ImagingPalette;
 
 #define IMAGING_MODE_LENGTH 6+1 /* Band names ("1", "L", "P", "RGB", "RGBA", "CMYK", "YCbCr", "BGR;xy") */
 
+typedef struct {
+    char *ptr;
+    int size;
+} ImagingMemoryBlock;
+
 struct ImagingMemoryInstance {
 
     /* Format */
-    char mode[IMAGING_MODE_LENGTH];	/* Band names ("1", "L", "P", "RGB", "RGBA", "CMYK", "YCbCr", "BGR;xy") */
-    int type;		/* Data type (IMAGING_TYPE_*) */
-    int depth;		/* Depth (ignored in this version) */
-    int bands;		/* Number of bands (1, 2, 3, or 4) */
-    int xsize;		/* Image dimension. */
+    char mode[IMAGING_MODE_LENGTH];     /* Band names ("1", "L", "P", "RGB", "RGBA", "CMYK", "YCbCr", "BGR;xy") */
+    int type;           /* Data type (IMAGING_TYPE_*) */
+    int depth;          /* Depth (ignored in this version) */
+    int bands;          /* Number of bands (1, 2, 3, or 4) */
+    int xsize;          /* Image dimension. */
     int ysize;
 
     /* Colour palette (for "P" images only) */
     ImagingPalette palette;
 
     /* Data pointers */
-    UINT8 **image8;	/* Set for 8-bit images (pixelsize=1). */
-    INT32 **image32;	/* Set for 32-bit images (pixelsize=4). */
+    UINT8 **image8;     /* Set for 8-bit images (pixelsize=1). */
+    INT32 **image32;    /* Set for 32-bit images (pixelsize=4). */
 
     /* Internals */
-    char **image;	/* Actual raster data. */
-    char *block;	/* Set if data is allocated in a single block. */
+    char **image;       /* Actual raster data. */
+    char *block;        /* Set if data is allocated in a single block. */
+    ImagingMemoryBlock *blocks;     /* Memory blocks for pixel storage */
 
-    int pixelsize;	/* Size of a pixel, in bytes (1, 2 or 4) */
-    int linesize;	/* Size of a line, in bytes (xsize * pixelsize) */
+    int pixelsize;      /* Size of a pixel, in bytes (1, 2 or 4) */
+    int linesize;       /* Size of a line, in bytes (xsize * pixelsize) */
 
     /* Virtual methods */
     void (*destroy)(Imaging im);
@@ -130,11 +136,11 @@ struct ImagingAccessInstance {
 struct ImagingHistogramInstance {
 
     /* Format */
-    char mode[IMAGING_MODE_LENGTH];	/* Band names (of corresponding source image) */
-    int bands;		/* Number of bands (1, 3, or 4) */
+    char mode[IMAGING_MODE_LENGTH];     /* Band names (of corresponding source image) */
+    int bands;          /* Number of bands (1, 3, or 4) */
 
     /* Data */
-    long *histogram;	/* Histogram (bands*256 longs) */
+    long *histogram;    /* Histogram (bands*256 longs) */
 
 };
 
@@ -142,39 +148,53 @@ struct ImagingHistogramInstance {
 struct ImagingPaletteInstance {
 
     /* Format */
-    char mode[IMAGING_MODE_LENGTH];	/* Band names */
+    char mode[IMAGING_MODE_LENGTH];     /* Band names */
 
     /* Data */
     UINT8 palette[1024];/* Palette data (same format as image data) */
 
-    INT16* cache;	/* Palette cache (used for predefined palettes) */
-    int keep_cache;	/* This palette will be reused; keep cache */
+    INT16* cache;       /* Palette cache (used for predefined palettes) */
+    int keep_cache;     /* This palette will be reused; keep cache */
 
 };
 
+typedef struct ImagingMemoryArena {
+    int alignment;        /* Alignment in memory of each line of an image */
+    int block_size;       /* Preferred block size, bytes */
+    int blocks_max;       /* Maximum number of cached blocks */
+    int blocks_cached;    /* Current number of blocks not associated with images */
+    ImagingMemoryBlock *blocks_pool;
+    int stats_new_count;           /* Number of new allocated images */
+    int stats_allocated_blocks;    /* Number of allocated blocks */
+    int stats_reused_blocks;       /* Number of blocks which were retrieved from a pool */
+    int stats_reallocated_blocks;  /* Number of blocks which were actually reallocated after retrieving */
+    int stats_freed_blocks;        /* Number of freed blocks */
+} *ImagingMemoryArena;
+
 
 /* Objects */
 /* ------- */
 
-extern int ImagingNewCount;
+extern struct ImagingMemoryArena ImagingDefaultArena;
+extern int ImagingMemorySetBlocksMax(ImagingMemoryArena arena, int blocks_max);
+extern void ImagingMemoryClearCache(ImagingMemoryArena arena, int new_size);
 
 extern Imaging ImagingNew(const char* mode, int xsize, int ysize);
-extern Imaging ImagingNew2(const char* mode, Imaging imOut, Imaging imIn);
+extern Imaging ImagingNewDirty(const char* mode, int xsize, int ysize);
+extern Imaging ImagingNew2Dirty(const char* mode, Imaging imOut, Imaging imIn);
 extern void    ImagingDelete(Imaging im);
 
 extern Imaging ImagingNewBlock(const char* mode, int xsize, int ysize);
-extern Imaging ImagingNewArray(const char* mode, int xsize, int ysize);
 extern Imaging ImagingNewMap(const char* filename, int readonly,
                              const char* mode, int xsize, int ysize);
 
 extern Imaging ImagingNewPrologue(const char *mode,
                                   int xsize, int ysize);
 extern Imaging ImagingNewPrologueSubtype(const char *mode,
-                                  int xsize, int ysize,
-                                  int structure_size);
-extern Imaging ImagingNewEpilogue(Imaging im);
+                                         int xsize, int ysize,
+                                         int structure_size);
 
-extern void ImagingCopyInfo(Imaging destination, Imaging source);
+extern void ImagingCopyPalette(Imaging destination, Imaging source);
 
 extern void ImagingHistogramDelete(ImagingHistogram histogram);
 
@@ -182,8 +202,6 @@ extern void ImagingAccessInit(void);
 extern ImagingAccess ImagingAccessNew(Imaging im);
 extern void _ImagingAccessDelete(Imaging im, ImagingAccess access);
 #define ImagingAccessDelete(im, access) /* nop, for now */
-/*#define ImagingAccessDelete(im, access) \
-  ((access)->dynamic ? _ImagingAccessDelete((im), (access)), 0 : 0)) */
 
 extern ImagingPalette ImagingPaletteNew(const char *mode);
 extern ImagingPalette ImagingPaletteNewBrowser(void);
@@ -192,10 +210,10 @@ extern void           ImagingPaletteDelete(ImagingPalette palette);
 
 extern int  ImagingPaletteCachePrepare(ImagingPalette palette);
 extern void ImagingPaletteCacheUpdate(ImagingPalette palette,
-				      int r, int g, int b);
+                                      int r, int g, int b);
 extern void ImagingPaletteCacheDelete(ImagingPalette palette);
 
-#define	ImagingPaletteCache(p, r, g, b)\
+#define ImagingPaletteCache(p, r, g, b)\
     p->cache[(r>>2) + (g>>2)*64 + (b>>2)*64*64]
 
 extern Imaging ImagingQuantize(Imaging im, int colours, int mode, int kmeans);
@@ -261,16 +279,18 @@ extern Imaging ImagingFillLinearGradient(const char* mode);
 extern Imaging ImagingFillRadialGradient(const char* mode);
 extern Imaging ImagingFilter(
     Imaging im, int xsize, int ysize, const FLOAT32* kernel,
-    FLOAT32 offset, FLOAT32 divisor);
+    FLOAT32 offset);
 extern Imaging ImagingFlipLeftRight(Imaging imOut, Imaging imIn);
 extern Imaging ImagingFlipTopBottom(Imaging imOut, Imaging imIn);
 extern Imaging ImagingGaussianBlur(Imaging imOut, Imaging imIn, float radius,
-    int passes);
+                                   int passes);
 extern Imaging ImagingGetBand(Imaging im, int band);
+extern Imaging ImagingMerge(const char* mode, Imaging bands[4]);
+extern int ImagingSplit(Imaging im, Imaging bands[4]);
 extern int ImagingGetBBox(Imaging im, int bbox[4]);
 typedef struct { int x, y; INT32 count; INT32 pixel; } ImagingColorItem;
 extern ImagingColorItem* ImagingGetColors(Imaging im, int maxcolors,
-    int *colors);
+                                          int *colors);
 extern int ImagingGetExtrema(Imaging im, void *extrema);
 extern int ImagingGetProjection(Imaging im, UINT8* xproj, UINT8* yproj);
 extern ImagingHistogram ImagingGetHistogram(
@@ -290,14 +310,17 @@ extern Imaging ImagingRankFilter(Imaging im, int size, int rank);
 extern Imaging ImagingRotate90(Imaging imOut, Imaging imIn);
 extern Imaging ImagingRotate180(Imaging imOut, Imaging imIn);
 extern Imaging ImagingRotate270(Imaging imOut, Imaging imIn);
-extern Imaging ImagingResample(Imaging imIn, int xsize, int ysize, int filter);
 extern Imaging ImagingTranspose(Imaging imOut, Imaging imIn);
+extern Imaging ImagingTransverse(Imaging imOut, Imaging imIn);
+extern Imaging ImagingResample(Imaging imIn, int xsize, int ysize, int filter, float box[4]);
 extern Imaging ImagingTransform(
     Imaging imOut, Imaging imIn, int method, int x0, int y0, int x1, int y1,
     double *a, int filter, int fill);
 extern Imaging ImagingUnsharpMask(
     Imaging imOut, Imaging im, float radius, int percent, int threshold);
 extern Imaging ImagingBoxBlur(Imaging imOut, Imaging imIn, float radius, int n);
+extern Imaging ImagingColorLUT3D_linear(Imaging imOut, Imaging imIn,
+    int table_channels, int size1D, int size2D, int size3D, INT16* table);
 
 extern Imaging ImagingCopy2(Imaging imOut, Imaging imIn);
 extern Imaging ImagingConvert2(Imaging imOut, Imaging imIn);
@@ -335,7 +358,7 @@ extern int ImagingDrawChord(Imaging im, int x0, int y0, int x1, int y1,
 extern int ImagingDrawEllipse(Imaging im, int x0, int y0, int x1, int y1,
                               const void* ink, int fill, int op);
 extern int ImagingDrawLine(Imaging im, int x0, int y0, int x1, int y1,
-			   const void* ink, int op);
+                           const void* ink, int op);
 extern int ImagingDrawWideLine(Imaging im, int x0, int y0, int x1, int y1,
                                const void* ink, int width, int op);
 extern int ImagingDrawPieslice(Imaging im, int x0, int y0, int x1, int y1,
@@ -343,9 +366,9 @@ extern int ImagingDrawPieslice(Imaging im, int x0, int y0, int x1, int y1,
                                int op);
 extern int ImagingDrawPoint(Imaging im, int x, int y, const void* ink, int op);
 extern int ImagingDrawPolygon(Imaging im, int points, int *xy,
-			      const void* ink, int fill, int op);
+                              const void* ink, int fill, int op);
 extern int ImagingDrawRectangle(Imaging im, int x0, int y0, int x1, int y1,
-				const void* ink, int fill, int op);
+                                const void* ink, int fill, int op);
 
 /* Level 2 graphics (WORK IN PROGRESS) */
 extern ImagingOutline ImagingOutlineNew(void);
@@ -357,7 +380,7 @@ extern int ImagingDrawOutline(Imaging im, ImagingOutline outline,
 extern int ImagingOutlineMove(ImagingOutline outline, float x, float y);
 extern int ImagingOutlineLine(ImagingOutline outline, float x, float y);
 extern int ImagingOutlineCurve(ImagingOutline outline, float x1, float y1,
-                                float x2, float y2, float x3, float y3);
+                               float x2, float y2, float x3, float y3);
 extern int ImagingOutlineTransform(ImagingOutline outline, double a[6]);
 
 extern int ImagingOutlineClose(ImagingOutline outline);
@@ -386,29 +409,30 @@ extern UINT32 ImagingCRC32(UINT32 crc, UINT8* buffer, int bytes);
 /* Codecs */
 typedef struct ImagingCodecStateInstance *ImagingCodecState;
 typedef int (*ImagingCodec)(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 
 extern int ImagingBcnDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingBitDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingEpsEncode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingFliDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingGifDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingGifEncode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingHexDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
-#ifdef	HAVE_LIBJPEG
+                            UINT8* buffer, int bytes);
+#ifdef  HAVE_LIBJPEG
 extern int ImagingJpegDecode(Imaging im, ImagingCodecState state,
-			     UINT8* buffer, int bytes);
+                             UINT8* buffer, int bytes);
 extern int ImagingJpegDecodeCleanup(ImagingCodecState state);
+extern int ImagingJpegUseJCSExtensions(void);
 
 extern int ImagingJpegEncode(Imaging im, ImagingCodecState state,
-			     UINT8* buffer, int bytes);
+                             UINT8* buffer, int bytes);
 #endif
 #ifdef HAVE_OPENJPEG
 extern int ImagingJpeg2KDecode(Imaging im, ImagingCodecState state,
@@ -418,58 +442,58 @@ extern int ImagingJpeg2KEncode(Imaging im, ImagingCodecState state,
                                UINT8* buffer, int bytes);
 extern int ImagingJpeg2KEncodeCleanup(ImagingCodecState state);
 #endif
-extern int ImagingLzwDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
-#ifdef	HAVE_LIBTIFF
+#ifdef  HAVE_LIBTIFF
 extern int ImagingLibTiffDecode(Imaging im, ImagingCodecState state,
-				UINT8* buffer, int bytes);
+                                UINT8* buffer, int bytes);
 extern int ImagingLibTiffEncode(Imaging im, ImagingCodecState state,
-				UINT8* buffer, int bytes);
+                                UINT8* buffer, int bytes);
 #endif
-#ifdef	HAVE_LIBMPEG
+#ifdef  HAVE_LIBMPEG
 extern int ImagingMpegDecode(Imaging im, ImagingCodecState state,
-			     UINT8* buffer, int bytes);
+                             UINT8* buffer, int bytes);
 #endif
 extern int ImagingMspDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingPackbitsDecode(Imaging im, ImagingCodecState state,
-				 UINT8* buffer, int bytes);
+                                 UINT8* buffer, int bytes);
 extern int ImagingPcdDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingPcxDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingPcxEncode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingRawDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingRawEncode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
+extern int ImagingSgiRleDecode(Imaging im, ImagingCodecState state,
+                               UINT8* buffer, int bytes);
+extern int ImagingSgiRleDecodeCleanup(ImagingCodecState state);
 extern int ImagingSunRleDecode(Imaging im, ImagingCodecState state,
-			       UINT8* buffer, int bytes);
+                               UINT8* buffer, int bytes);
 extern int ImagingTgaRleDecode(Imaging im, ImagingCodecState state,
-			       UINT8* buffer, int bytes);
+                               UINT8* buffer, int bytes);
+extern int ImagingTgaRleEncode(Imaging im, ImagingCodecState state,
+                               UINT8* buffer, int bytes);
 extern int ImagingXbmDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingXbmEncode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
-#ifdef	HAVE_LIBZ
+                            UINT8* buffer, int bytes);
+#ifdef  HAVE_LIBZ
 extern int ImagingZipDecode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
+extern int ImagingZipDecodeCleanup(ImagingCodecState state);
 extern int ImagingZipEncode(Imaging im, ImagingCodecState state,
-			    UINT8* buffer, int bytes);
+                            UINT8* buffer, int bytes);
 extern int ImagingZipEncodeCleanup(ImagingCodecState state);
 #endif
 
 typedef void (*ImagingShuffler)(UINT8* out, const UINT8* in, int pixels);
 
 /* Public shufflers */
-extern void ImagingPackRGB(UINT8* out, const UINT8* in, int pixels);
 extern void ImagingPackBGR(UINT8* out, const UINT8* in, int pixels);
-extern void ImagingUnpackRGB(UINT8* out, const UINT8* in, int pixels);
-extern void ImagingUnpackBGR(UINT8* out, const UINT8* in, int pixels);
 extern void ImagingUnpackYCC(UINT8* out, const UINT8* in, int pixels);
 extern void ImagingUnpackYCCA(UINT8* out, const UINT8* in, int pixels);
-extern void ImagingUnpackYCbCr(UINT8* out, const UINT8* in, int pixels);
 
 extern void ImagingConvertRGB2YCbCr(UINT8* out, const UINT8* in, int pixels);
 extern void ImagingConvertYCbCr2RGB(UINT8* out, const UINT8* in, int pixels);
@@ -504,12 +528,18 @@ extern Py_ssize_t _imaging_tell_pyFd(PyObject *fd);
 
 
 /* Errcodes */
-#define	IMAGING_CODEC_END	 1
-#define	IMAGING_CODEC_OVERRUN	-1
-#define	IMAGING_CODEC_BROKEN	-2
-#define	IMAGING_CODEC_UNKNOWN	-3
-#define	IMAGING_CODEC_CONFIG	-8
-#define	IMAGING_CODEC_MEMORY	-9
+#define IMAGING_CODEC_END        1
+#define IMAGING_CODEC_OVERRUN   -1
+#define IMAGING_CODEC_BROKEN    -2
+#define IMAGING_CODEC_UNKNOWN   -3
+#define IMAGING_CODEC_CONFIG    -8
+#define IMAGING_CODEC_MEMORY    -9
+
+
+
+#include "ImagingUtils.h"
+extern UINT8 *clip8_lookups;
+
 
 #if defined(__cplusplus)
 }
